/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.hadoop.hive.ql.parse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.apache.hadoop.hive.ql.exec.Task;
import org.apache.hadoop.hive.ql.hooks.Entity;
import org.apache.hadoop.hive.ql.hooks.ReadEntity;
import org.apache.hadoop.hive.ql.hooks.WriteEntity;
import org.apache.hadoop.hive.ql.metadata.Table;

import java.io.Serializable;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class ParentTableAuthorizationHook extends AbstractSemanticAnalyzerHook{

  private static final Log LOG = LogFactory.getLog(ParentTableAuthorizationHook.class);

  @Override
  public void postAnalyze(HiveSemanticAnalyzerHookContext context, List<Task<? extends Serializable>> rootTasks) throws SemanticException {
    LOG.info("ParentTableAuthorizationHook.postAnalyze called");
    int sz = context.getInputs().size();
    Set<ReadEntity> additionalInputs = new HashSet<ReadEntity>();
    for (Entity e : context.getInputs()) {
      if (e.getType() == Entity.Type.PARTITION) {
        additionalInputs.add(new ReadEntity(e.getTable()));
      }
    }
    context.getInputs().addAll(additionalInputs);

    LOG.debug("ParentTableAuthorizationHook.postAnalyze inputs done, pre-count:" + sz
        + ", post-count:"+context.getInputs().size()+", tables:"+additionalInputs.size());

    sz = context.getOutputs().size();

    Set<WriteEntity> additionalOutputs = new HashSet<WriteEntity>();
    for (Entity e : context.getOutputs()) {
      if (e.getType() == Entity.Type.PARTITION) {
        additionalOutputs.add(new WriteEntity(e.getTable(), WriteEntity.WriteType.DDL_NO_LOCK));
      }
    }
    context.getOutputs().addAll(additionalOutputs);

    LOG.debug("ParentTableAuthorizationHook.postAnalyze outputs done, pre-count:" + sz
        + ", post-count:"+context.getOutputs().size()+", tables:"+additionalOutputs.size());
  }

}
